@@ -2,18 +2,19 @@
 #define	cmlmbounce_h
 
 /*
-** Copyright 2000 Double Precision, Inc.
+** Copyright 2000-2007 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
-static const char cmlmbounce_h_rcsid[]="$Id: cmlmbounce.h,v 1.2 2000/09/16 21:37:20 mrsam Exp $";
+static const char cmlmbounce_h_rcsid[]="$Id: cmlmbounce.h,v 1.3 2007/03/01 04:32:03 mrsam Exp $";
 
 #include	"config.h"
-#include	"afx/afx.h"
+
+#include	<string>
 
 int dobounce(const char *);
 int dobounce1(const char *);
 int dobounce2(const char *);
-int bouncewarning(CString, CString, CString);
+int bouncewarning(std::string, std::string, std::string);
 
 #endif
