@@ -6,13 +6,13 @@
 ** See COPYING for distribution information.
 */
 
-static const char cmlmmoderate_h_rcsid[]="$Id: cmlmmoderate.h,v 1.4 2001/08/05 20:00:33 mrsam Exp $";
+static const char cmlmmoderate_h_rcsid[]="$Id: cmlmmoderate.h,v 1.5 2007/03/01 04:32:03 mrsam Exp $";
 
 #include	"config.h"
 #include	<iostream>
 
 int sendinitmod(afxipipestream &, const char *, const char *);
 int cmdmoderate();
-int postmsg(istream &, int (*)(istream &, ostream &));
+int postmsg(std::istream &, int (*)(std::istream &, std::ostream &));
 
 #endif
