@@ -7,7 +7,7 @@
 #define	comverp_h
 
 #if	HAVE_CONFIG_H
-#include	"config.h"
+#include	"libs/courier_lib_config.h"
 #endif
 
 #include	<stdlib.h>
