@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2007 Double Precision, Inc.
+** Copyright 1998 - 2008 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -41,7 +41,7 @@
 #include	<courierauth.h>
 #include	<courierauthsasl.h>
 
-static const char rcsid[]="$Id: courieresmtpd.c,v 1.53 2007/01/06 02:12:56 mrsam Exp $";
+static const char rcsid[]="$Id: courieresmtpd.c,v 1.54 2008/06/30 00:31:15 mrsam Exp $";
 
 static char helobuf[256];
 static char authuserbuf[256];
@@ -67,6 +67,18 @@ static const char *tcpremoteip, *tcpremotehost;
 
 static time_t teergrube=INIT_TEERGRUBE;
 
+extern const char *externalauth();
+
+static const char *smtp_externalauth()
+{
+	const char *p;
+
+	if ((p=getenv("ESMTP_TLS")) && atoi(p))
+		return externalauth();
+
+	return NULL;
+}
+
 static void tarpit()
 {
 	sleep(teergrube);
@@ -139,7 +151,7 @@ EHLO_SECURITY_EXTENSION
 "250-SIZE\r\n"
 "250 DSN\r\n";
 const char *me=config_me();
-const char *p;
+const char *p, *q;
 
 	if (helobuf[0] == 0) 
 	{
@@ -174,20 +186,31 @@ const char *p;
 	else
 		p=getenv("ESMTPAUTH");
 
-	if (p && *p)
+	if (!p)
+		p="";
+
+	q=smtp_externalauth() ? "EXTERNAL":"";
+
+	if (*p || *q)
 	{
 		addiovec("250-AUTH ", 9);
 		addiovec(p, strlen(p));
+		if (*p && *q)
+			addiovec(" ", 1);
+		addiovec(q, strlen(q));
 		addiovec("\r\n", 2);
-
+#if 0
 		/*
 		** Netscape.
 		*/
 
 		addiovec("250-AUTH=", 9);
 		addiovec(p, strlen(p));
+		if (*p && *q)
+			addiovec(" ", 1);
+		addiovec(q, strlen(q));
 		addiovec(" X-NETSCAPE-HAS-BUGS\r\n", 22);
-
+#endif
 	}
 
 	if (hastls)
@@ -1152,6 +1175,7 @@ int	authenticated=0;
 				       "bits");
 			strncat(strcat(tlsbuf, ","), cinfo.cipher, 40);
 
+			couriertls_export_subject_environment(&cinfo);
 			couriertls_destroy(&cinfo);
 
 			tls=1;
@@ -1168,6 +1192,8 @@ int	authenticated=0;
 			close(pipefd[0]);
 			cancelsubmit();
 			startsubmit(tls);	/* Mark */
+
+			setenv("ESMTP_TLS", "1", 1);
 			continue;
 		}
 
@@ -1206,13 +1232,22 @@ int	authenticated=0;
 				continue;
 			}
 
+			if (initreply && *initreply)
+			{
+				if (strcmp(initreply, "=") == 0)
+					initreply="";
+			}
+			else
+			{
+				initreply=0;
+			}
 
-			if (auth_sasl(authmethod, initreply && *initreply
-				      ? initreply:NULL,
-				      sasl_conv_func,
-				      NULL,
-				      &authtype,
-				      &authdata) != AUTHSASL_OK)
+			if (auth_sasl_ex(authmethod, initreply,
+					 smtp_externalauth(),
+					 sasl_conv_func,
+					 NULL,
+					 &authtype,
+					 &authdata) != AUTHSASL_OK)
 			{
 				/* TODO - better error messages */
 
