@@ -2,9 +2,9 @@
 #
 # TITLE: LDAP-based aliasing/routing
 #
-# $Id: admin-15ldapa.pl,v 1.3 2001/07/08 02:07:41 mrsam Exp $
+# $Id: admin-15ldapa.pl,v 1.5 2004/10/24 02:01:55 mrsam Exp $
 #
-# Copyright 2001 Double Precision, Inc.  See COPYING for
+# Copyright 2001-2004 Double Precision, Inc.  See COPYING for
 # distribution information.
 
 use webadmin;
@@ -12,8 +12,7 @@ use webadmin;
 my $errstr="";
 
 display_form("notsupp.html")
-    unless -f "$libexecdir/authlib/authldap" ||
-    -f "$libexecdir/authlib/authdaemond.ldap";
+    unless -f "$authldaprc";
 
 if ($cgi->param("Save"))
 {
@@ -44,7 +43,7 @@ my $ldap=ReadKWConfigFile("ldapaliasrc");
 display_form("admin-15ldapa.html",
 	     {
 		 "ERROR" => $errstr,
-		 "ENABLED" => ("<input type=checkbox name=ENABLED" . ($$ldap{'LDAP_ALIAS'} ? " checked":"")) . ">",
+		 "ENABLED" => ("<input type=\"checkbox\" name=\"ENABLED\"" . ($$ldap{'LDAP_ALIAS'} ? " checked=\"checked\"":"")) . " />",
 
 		 "SERVER" => $$ldap{'LDAP_SERVER'},
 		 "PORT" => $$ldap{'LDAP_PORT'},
