@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2004 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -48,7 +48,7 @@
 #include	<ctype.h>
 #include	<numlib/numlib.h>
 
-static const char rcsid[]="$Id: maildirquota.c,v 1.25 2004/06/18 23:43:40 mrsam Exp $";
+static const char rcsid[]="$Id: maildirquota.c,v 1.30 2006/05/28 15:29:52 mrsam Exp $";
 
 static void parsequotastr(const char *, struct maildirquota *);
 
@@ -488,8 +488,8 @@ static int docheckquota(struct maildirsize *info,
 	if (doaddquota(info, maildirsize_fd, maildirsize_size,
 		maildirsize_cnt, 1))
 	{
-		free(newmaildirsizename);
 		unlink(newmaildirsizename);
+		free(newmaildirsizename);
 		close(maildirsize_fd);
 		errno=EIO;
 		return (-1);
@@ -691,7 +691,11 @@ char	*p;
 int	n;
 
 	if ( *subdir != '.' || strcmp(subdir, ".") == 0 ||
-		strcmp(subdir, "..") == 0 || strcmp(subdir, "." TRASH) == 0)
+		strcmp(subdir, "..") == 0
+#ifndef TRASHQUOTA
+				|| strcmp(subdir, "." TRASH) == 0
+#endif
+		)
 		return (0);
 
 	p=(char *)malloc(strlen(dir)+strlen(subdir)+2);
@@ -978,7 +982,7 @@ void maildir_quota_set(const char *dir, const char *quota)
 
 static void do_deliver_warning(const char *msgfile, const char *dir)
 {
-int	fdin, fd;
+int	fdin, fd, ret;
 FILE *fpout;
 time_t	t;
 size_t	l, msg_len;
@@ -1004,7 +1008,9 @@ struct maildir_tmpcreate_info createInfo;
 
 	strcat(strcpy(qname, dir), "/quotawarn");
 	time(&t);
-	if (stat(qname, &sb) == 0 && ((sb.st_mtime + 86400) > t))
+	ret = stat(qname, &sb);
+	if ((ret == -1 && errno != ENOENT) ||
+	    (ret == 0 && (sb.st_mtime + 86400) > t))
 	{
 		free(qname);
 		close(fdin);
@@ -1013,12 +1019,14 @@ struct maildir_tmpcreate_info createInfo;
 
 	fd = open(qname, O_WRONLY|O_CREAT|O_TRUNC, 0644);
 	free(qname);
-	if (!fd)
+	if (fd == -1)
 	{
 		close(fdin);
 		return;
 	}
-	write(fd, buf, 1);
+	if (write(fd, buf, 1) < 0)
+		perror(msgfile);
+
 	close(fd);
 
 	strcpy(buf, "Date: ");
@@ -1028,8 +1036,8 @@ struct maildir_tmpcreate_info createInfo;
 	hostname[0]=0;
 	hostname[sizeof(hostname)-1]=0;
 	gethostname(hostname, sizeof(hostname)-1);
-	sprintf(buf, "%sMessage-Id: <%lu.overquota@%-1.256s>\n",
-		buf, (unsigned long)t, hostname);
+	sprintf(buf+strlen(buf), "Message-Id: <%lu.overquota@%-1.256s>\n",
+		(unsigned long)t, hostname);
 
 	if (stat(msgfile, &sb) < 0) {
 		close(fdin);
@@ -1055,7 +1063,11 @@ struct maildir_tmpcreate_info createInfo;
 
 	while ((n=read(fdin, buf, sizeof(buf))) > 0)
 	{
-		fwrite(buf, n, 1, fpout);
+		if (fwrite(buf, n, 1, fpout) != 1)
+		{
+			perror(createInfo.tmpname);
+			break;
+		}
 	}
 	close(fdin);
 
