@@ -8,7 +8,7 @@
 #include	<errno.h>
 #include	"rfc1035_res.h"
 
-static const char rcsid[]="$Id: rfc1035qptr.c,v 1.9 2004/08/15 16:41:02 mrsam Exp $";
+static const char rcsid[]="$Id: rfc1035qptr.c,v 1.10 2005/06/24 17:47:33 mrsam Exp $";
 
 static int ptr(struct rfc1035_res *, const void *, int,
 	       void (*)(const char *, void *),
@@ -114,7 +114,7 @@ char	name[256], ptrbuf[256];
 	}
 
 	if (rfc1035_resolve_cname(res, RFC1035_RESOLVE_RECURSIVE, name,
-		RFC1035_TYPE_PTR, RFC1035_CLASS_IN, &reply) < 0 ||
+		RFC1035_TYPE_PTR, RFC1035_CLASS_IN, &reply, 0) < 0 ||
 		reply == 0 ||
 		(n=rfc1035_replysearch_an( res, reply, name, RFC1035_TYPE_PTR,
 			RFC1035_CLASS_IN, 0)) < 0 ||
