@@ -1,18 +1,18 @@
 /*
-** Copyright 2002 Double Precision, Inc.
+** Copyright 2002-2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
 #define TLSCACHEMINSIZE (sizeof(struct hdr) + 5 * (sizeof(struct obj)+8))
 #include "tlscache.c"
 
-static const char tlscachetest_rcsid[]="$Id: tlscachetest.c,v 1.2 2002/07/12 15:11:09 mrsam Exp $";
+static const char tlscachetest_rcsid[]="$Id: tlscachetest.c,v 1.3 2006/05/28 15:29:52 mrsam Exp $";
 
 static int printcache(void *rec, size_t recsize, int *doupdate,
 		      void *arg)
 {
-	fwrite((const char *)rec, recsize, 1, stdout);
-	printf("\n");
+	if (fwrite((const char *)rec, recsize, 1, stdout) == 1)
+		printf("\n");
 	return 0;
 }
 
