@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: auth.c,v 1.41 2006/05/28 15:29:52 mrsam Exp $
+** $Id: auth.c,v 1.43 2006/11/19 19:02:27 mrsam Exp $
 */
 
 #include	"sqwebmail.h"
@@ -96,6 +96,7 @@ static int doauthlogin(struct authinfo *a, void *vp)
 	static char *authaddr=NULL;
 	static char *authfullname=NULL;
 	static char *authoptions=NULL;
+	static char *authenticated=NULL;
 	const char *n;
 	char *b;
 	int rc;
@@ -153,6 +154,19 @@ static int doauthlogin(struct authinfo *a, void *vp)
 		free(authoptions);
 	authoptions=b;
 
+	n=a->address;
+	if (!n) n="";
+
+	b=malloc(sizeof("AUTHENTICATED=")+strlen(n));
+
+	if (!b)
+		enomem();
+	strcat(strcpy(b, "AUTHENTICATED="), n);
+	putenv(b);
+	if (authenticated)
+		free(authenticated);
+	authenticated=b;
+
 	return (0);
 }
 
@@ -210,17 +224,21 @@ int prelogin(const char *u)
 
 const char *login_returnaddr()
 {
-	const char *p=getenv("AUTHADDR");
-	const char *domain=myhostname();
-
 	static char *addrbuf=0;
-
+	const char *p, *domain="";
+	
+	if ((p=getenv("AUTHENTICATED")) == NULL || *p == 0)
+		p=getenv("AUTHADDR");
 	if (!p)	p="";
+	if (strchr(p, '@') == 0)
+		domain=myhostname();
+
 	if (addrbuf)	free(addrbuf);
 	addrbuf=malloc(strlen(domain)+strlen(p)+2);
 	if (!addrbuf)	enomem();
+
 	strcpy(addrbuf, p);
-	if (strchr(addrbuf, '@') == 0)
+	if (*domain)
 		strcat(strcat(addrbuf, "@"), domain);
 	return (addrbuf);
 }
