@@ -1,5 +1,5 @@
 /*
-** $Id: buf.h,v 1.3 1999/12/08 06:00:38 mrsam Exp $
+** $Id: buf.h,v 1.4 2007/05/20 00:04:55 mrsam Exp $
 */
 #ifndef	buf_h
 #define	buf_h
@@ -38,9 +38,6 @@ void	buf_catn(struct buf *, const char *, size_t);
 void	buf_memcpy(struct buf *, const char *, size_t);
 void	buf_memcat(struct buf *, const char *, size_t);
 void	buf_trimleft(struct buf *, size_t);
+void	buf_append(struct buf *, char c);
 
-void	buf_allocbuf(struct buf *b, size_t n);
-
-#define	buf_append(p, c) do { if ((p)->cnt >= (p)->size) buf_allocbuf((p), \
-			(p)->cnt+1); (p)->ptr[(p)->cnt++]=(c); } while (0)
 #endif
