@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: folder.c,v 1.179 2007/05/20 00:04:55 mrsam Exp $
+** $Id: folder.c,v 1.182 2007/09/23 13:20:11 mrsam Exp $
 */
 #include	"config.h"
 #include	<stdio.h>
@@ -253,6 +253,7 @@ void folder_delmsgs(const char *dir, size_t pos)
 {
 	int	rc=0;
 	char	buf[2];
+	char	*cur;
 
 	strcpy(buf, ACL_DELETEMSGS);
 	acl_computeRightsOnFolder(dir, buf);
@@ -289,7 +290,7 @@ void folder_delmsgs(const char *dir, size_t pos)
 		return;
 	    }
 	    
-	    char *cur = malloc(strlen(deldir)+5);
+	    cur = malloc(strlen(deldir)+5);
 	    strcpy(cur, deldir);
 	    strcat(cur, "/cur");
 
@@ -1850,13 +1851,22 @@ const char *redirect_hash(const char *timestamp)
 	struct stat stat_buf;
 
 	char buffer[NUMBUFSIZE*2+10];
+	const char *p=getenv("SQWEBMAIL_RANDSEED");
 
-	if (strlen(timestamp) >= NUMBUFSIZE
-	    || stat(SENDITSH, &stat_buf) < 0)
+	if (strlen(timestamp) >= NUMBUFSIZE)
 		return "";
 
 	strcat(strcpy(buffer, timestamp), " ");
-	libmail_str_ino_t(stat_buf.st_ino, buffer+strlen(buffer));
+
+	if (p && *p)
+		strncat(buffer, p, NUMBUFSIZE);
+	else
+	{
+		if (stat(SENDITSH, &stat_buf) < 0)
+			return "";
+
+		libmail_str_ino_t(stat_buf.st_ino, buffer+strlen(buffer));
+	}
 
 	return md5_hash_courier(buffer);
 }
@@ -2880,7 +2890,7 @@ static void do_sharedhierlist(const char *folderdir,
 	}
 	free(url);
 	freeph(&ph);
-
+	printf("</table>\n");
 }
 
 static void do_folderlist(const char *inbox_pfix,
