@@ -1,5 +1,5 @@
 /*
-** $Id: pref.h,v 1.12 2005/11/19 21:57:29 mrsam Exp $
+** $Id: pref.h,v 1.13 2008/02/03 15:25:07 mrsam Exp $
 */
 #ifndef	pref_h
 #define	pref_h
@@ -19,6 +19,7 @@ extern void pref_pagesize();
 extern void pref_displayhtml();
 extern void pref_displayflowedtext();
 extern void pref_displaynoarchive();
+extern void pref_displaynoautorenamesent();
 extern void pref_displayweekstart();
 
 extern int pref_flagisoldest1st, pref_flagfullheaders;
@@ -28,6 +29,7 @@ extern int pref_autopurge;
 extern int pref_showhtml;
 extern int pref_noflowedtext;
 extern int pref_noarchive;
+extern int pref_noautorenamesent;
 extern int pref_startofweek;
 extern int pref_wikifmt;
 
