@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2006 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2008 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: sqwebmail.c,v 1.143 2007/09/03 15:44:10 mrsam Exp $
+** $Id: sqwebmail.c,v 1.144 2008/02/03 15:25:07 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"sqconfig.h"
@@ -1212,6 +1212,10 @@ static void do_output_form_loop(FILE *f)
 		{
 			pref_displaynoarchive();
 		}
+		else if (strcmp(kw, "NOAUTORENAMESENT") == 0)
+		{
+			pref_displaynoautorenamesent();
+		}
 		else if (strcmp(kw, "x") == 0)
 		{
 			pref_setprefs();
@@ -2059,7 +2063,7 @@ char	*cl=http11_best_content_language(templatedir,
 #endif
 }
 
-static void rename_sent_folder()
+void rename_sent_folder(int really)
 {
 	char buf[128];
 	char yyyymm[128];
@@ -2069,7 +2073,8 @@ static void rename_sent_folder()
 	struct tm *tm;
 	char *pp;
 
-	(void)maildir_create(INBOX "." SENT); /* No matter what */
+	if (really)
+		(void)maildir_create(INBOX "." SENT); /* No matter what */
 
 	time(&t);
 	tm=localtime(&t);
@@ -2096,9 +2101,11 @@ static void rename_sent_folder()
 
 	pp=folder_toutf7(buf);
 
-	rename("." SENT, pp);
+	if (really)
+		rename("." SENT, pp);
 	free(pp);
-	(void)maildir_create(INBOX "." SENT);
+	if (really)
+		(void)maildir_create(INBOX "." SENT);
 
 	write_sqconfig(".", SENTSTAMP, yyyymm);
 }
@@ -2555,18 +2562,13 @@ time_t	timeouthard=get_timeouthard();
 				setlang();
 #endif
 #endif
+				pref_init();
 				(void)maildir_create(INBOX "." DRAFTS);
-				{
-					const char *autorenamesent=AUTORENAMESENT;
-					const char *p=getenv("SQWEBMAIL_AUTORENAMESENT");
-					if (p && *p)
-						autorenamesent = p;
-					if ( strncmp(autorenamesent, "no", 2) != 0 )
-						(void)rename_sent_folder();
-				}
+
+				if (!pref_noautorenamesent)
+					(void)rename_sent_folder(1);
 				(void)maildir_create(INBOX "." SENT);
 				(void)maildir_create(INBOX "." TRASH);
-				pref_init();
 				maildir_autopurge();
 				unlink(SHAREDPATHCACHE);
 
